import boa
import pytest

import curve_dao


@pytest.fixture
def gauge_to_kill(etherscan_api_key):
    return boa.from_etherscan(
        "0x04e80db3f84873e4132b221831af1045d27f140f",
        name="LiquidityGaugeV6",
        api_key=etherscan_api_key,
    )


@pytest.fixture
def local_contract():
    return boa.loads(
        """
@external
def do_nothing():
    pass
    """
    )


@pytest.fixture
def actions(gauge_to_kill, local_contract):
    return [(gauge_to_kill.address, "set_killed", True), (local_contract, "do_nothing")]


@pytest.fixture
def description():
    return "test"


@pytest.fixture
def target():
    return curve_dao.addresses.CURVE_DAO_OWNERSHIP


@pytest.fixture
def evm_script(target, actions, etherscan_api_key):
    return curve_dao.prepare_evm_script(target, actions, etherscan_api_key)


def test_evm_script(evm_script, local_contract):
    assert (
        evm_script.hex()
        == f"0000000140907540d8a6c65c637785e8f8b742ae6b0b9968000000c4b61d27f600000000000000000000000004e80db3f84873e4132b221831af1045d27f140f00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000060000000000000000000000000000000000000000000000000000000000000002490b2299700000000000000000000000000000000000000000000000000000000000000010000000000000000000000000000000000000000000000000000000040907540d8a6c65c637785e8f8b742ae6b0b9968000000a4b61d27f6000000000000000000000000{local_contract.address[2:].lower()}000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000600000000000000000000000000000000000000000000000000000000000000004321f001300000000000000000000000000000000000000000000000000000000"
    )


@pytest.mark.ignore_isolation
@pytest.fixture
def test_create_vote(
    target, actions, description, etherscan_api_key, pinata_token, vote_creator
):
    with boa.env.prank(vote_creator):
        vote_id = curve_dao.create_vote(
            target, actions, description, etherscan_api_key, pinata_token
        )
    return vote_id


@pytest.mark.ignore_isolation
@pytest.fixture
def test_simulate(test_create_vote, target, etherscan_api_key):
    vote_id = test_create_vote
    assert curve_dao.simulate(vote_id, target["voting"], etherscan_api_key)


def test_gauge_killed(test_create_vote, test_simulate, gauge_to_kill):
    assert gauge_to_kill.is_killed() is True



import logging

import boa
from rich.logging import RichHandler
from rich.panel import Panel
from rich.pretty import Pretty

from .addresses import CONVEX_VOTERPROXY

logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("rich")


def simulate(vote_id: int, voting_contract: str, etherscan_api_key: str):
    logger.info("Simulating Vote")

    voting_contract = boa.from_etherscan(
        voting_contract, "AragonVoting", api_key=etherscan_api_key
    )

    # print vote details to console first:
    logger.info("Vote stats before Convex Vote:")
    vote_stats = Panel(Pretty(voting_contract.getVote(vote_id)))
    logger.info(vote_stats)

    # vote
    logger.info("Simulate Convex 'yes' vote")
    voting_contract.canVote(vote_id, CONVEX_VOTERPROXY)
    with boa.env.prank(CONVEX_VOTERPROXY):
        voting_contract.vote(vote_id, True, False)

    # sleep for a week so it has time to pass
    num_seconds = voting_contract.voteTime()
    boa.env.time_travel(seconds=num_seconds)

    # get vote stats:
    logger.info("Vote stats after 1 week:")
    vote_stats = Panel(Pretty(voting_contract.getVote(vote_id)))
    logger.info(vote_stats)

    # moment of truth - execute the vote!
    logger.info("Simulate proposal execution")
    assert voting_contract.canExecute(vote_id)
    with boa.env.prank(boa.env.generate_address()):
        voting_contract.executeVote(vote_id)

    logger.info("Vote Executed!")
    return True



import logging

import requests
from rich.logging import RichHandler

logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("rich")


def pin_to_ipfs(description: str, pinata_token: str):
    """Uploads vote description to IPFS via Pinata and returns the IPFS hash.

    NOTE: Needs environment variables for Pinata IPFS access. Please
    set up an IPFS project to generate API key and API secret!
    """

    url = "https://api.pinata.cloud/pinning/pinJSONToIPFS"
    headers = {
        "Authorization": f"Bearer {pinata_token}",
        "Content-Type": "application/json",
    }
    payload = {
        "pinataContent": {"text": description},
        "pinataMetadata": {"name": "pinnie.json"},
        "pinataOptions": {"cidVersion": 1},
    }

    response = requests.request("POST", url, json=payload, headers=headers)

    try:
        assert 200 <= response.status_code < 400
        ipfs_hash = response.json()["IpfsHash"]
        logger.info(f"Pinned Vote description to ipfs:{ipfs_hash}")
        return ipfs_hash

    except Exception:
        logger.exception(f"POST to IPFS failed: {response.status_code}")
        raise
